@@ -28,18 +28,33 @@
 #include "utils_internal.h"
 #include "logging_internal.h"
 #include "tscpp/api/noncopyable.h"
+#include "tscpp/api/Continuation.h"
 
 #ifndef INT64_MAX
 #define INT64_MAX (9223372036854775807LL)
 #endif
 
-using namespace atscppapi;
-using atscppapi::TransformationPlugin;
+namespace atscppapi
+{
+namespace detail
+{
+  class ResumeAfterPauseCont : public Continuation
+  {
+  public:
+    ResumeAfterPauseCont() : Continuation() {}
+
+    ResumeAfterPauseCont(Continuation::Mutex m) : Continuation(m) {}
+
+  protected:
+    int _run(TSEvent event, void *edata) override;
+  };
+
+} // end namespace detail
 
 /**
  * @private
  */
-struct atscppapi::TransformationPluginState : noncopyable {
+struct TransformationPluginState : noncopyable, public detail::ResumeAfterPauseCont {
   TSVConn vconn_;
   Transaction &transaction_;
   TransformationPlugin &transformation_plugin_;
@@ -50,12 +65,11 @@ struct atscppapi::TransformationPluginState : noncopyable {
   TSIOBufferReader output_buffer_reader_;
   int64_t bytes_written_;
   bool paused_;
-  TSCont resume_cont_;
 
   // We can only send a single WRITE_COMPLETE even though
   // we may receive an immediate event after we've sent a
   // write complete, so we'll keep track of whether or not we've
-  // sent the input end our write complte.
+  // sent the input end our write complete.
   bool input_complete_dispatched_;
 
   std::string request_xform_output_; // in case of request xform, data produced is buffered here
@@ -72,14 +86,13 @@ struct atscppapi::TransformationPluginState : noncopyable {
       output_buffer_reader_(nullptr),
       bytes_written_(0),
       paused_(false),
-      resume_cont_(nullptr),
       input_complete_dispatched_(false)
   {
     output_buffer_        = TSIOBufferCreate();
     output_buffer_reader_ = TSIOBufferReaderAlloc(output_buffer_);
   };
 
-  ~TransformationPluginState()
+  ~TransformationPluginState() override
   {
     if (output_buffer_reader_) {
       TSIOBufferReaderFree(output_buffer_reader_);
@@ -90,16 +103,17 @@ struct atscppapi::TransformationPluginState : noncopyable {
       TSIOBufferDestroy(output_buffer_);
       output_buffer_ = nullptr;
     }
-
-    // Cleanup pending cont
-    if (resume_cont_) {
-      TSContDataSet(resume_cont_, nullptr);
-    }
   }
 };
 
+} // end namespace atscppapi
+
+using namespace atscppapi;
+
 namespace
 {
+using ResumeAfterPauseCont = atscppapi::detail::ResumeAfterPauseCont;
+
 void
 cleanupTransformation(TSCont contp)
 {
@@ -249,7 +263,7 @@ handleTransformationPluginEvents(TSCont contp, TSEvent event, void *edata)
     return 0;
   }
 
-  // All other events includign WRITE_READY will just attempt to transform more data.
+  // All other events including WRITE_READY will just attempt to transform more data.
   return handleTransformationPluginRead(state->vconn_, state);
 }
 
@@ -273,31 +287,49 @@ TransformationPlugin::~TransformationPlugin()
   delete state_;
 }
 
-TSCont
+void
 TransformationPlugin::pause()
 {
-  if (state_->input_complete_dispatched_) {
+  if (state_->paused_) {
+    LOG_ERROR("Can not pause transformation, already paused  TransformationPlugin=%p (vconn)contp=%p tshttptxn=%p", this,
+              state_->vconn_, state_->txn_);
+  } else if (state_->input_complete_dispatched_) {
     LOG_ERROR("Can not pause transformation (transformation completed) TransformationPlugin=%p (vconn)contp=%p tshttptxn=%p", this,
               state_->vconn_, state_->txn_);
-    return nullptr;
   } else {
-    state_->paused_      = true;
-    state_->resume_cont_ = TSContCreate(&resumeCallback, TSContMutexGet(reinterpret_cast<TSCont>(state_->txn_)));
-    TSContDataSet(state_->resume_cont_, static_cast<void *>(state_));
-    return state_->resume_cont_;
+    state_->paused_ = true;
+    if (!static_cast<bool>(static_cast<ResumeAfterPauseCont *>(state_))) {
+      *static_cast<ResumeAfterPauseCont *>(state_) = ResumeAfterPauseCont(TSContMutexGet(reinterpret_cast<TSCont>(state_->txn_)));
+    }
   }
 }
 
-int
-TransformationPlugin::resumeCallback(TSCont cont, TSEvent event, void *edata)
+bool
+TransformationPlugin::isPaused() const
 {
-  auto state = static_cast<TransformationPluginState *>(TSContDataGet(cont));
-  if (state) {
-    state->paused_ = false;
-    handleTransformationPluginRead(state->vconn_, state);
-  }
+  return state_->paused_;
+}
+
+Continuation &
+TransformationPlugin::resumeCont()
+{
+  TSReleaseAssert(state_->paused_);
+
+  // The cast to a pointer to the intermediate base class ResumeAfterPauseCont is not strictly necessary.  It is
+  // possible that the transform plugin might want to defer work to other continuations in the future.  This would
+  // naturally result in TransactionPluginState having Continuation as an indirect base class multiple times, making
+  // disambiguation necessary when converting.
+  //
+  return *static_cast<ResumeAfterPauseCont *>(state_);
+}
+
+int
+ResumeAfterPauseCont::_run(TSEvent event, void *edata)
+{
+  auto state     = static_cast<TransformationPluginState *>(this);
+  state->paused_ = false;
+  handleTransformationPluginRead(state->vconn_, state);
 
-  TSContDestroy(cont);
   return TS_SUCCESS;
 }
 
@@ -394,7 +426,7 @@ TransformationPlugin::setOutputComplete()
               state_->txn_);
 
     // We're done without ever outputting anything, to correctly
-    // clean up we'll initiate a write then immeidately set it to 0 bytes done.
+    // clean up we'll initiate a write then immediately set it to 0 bytes done.
     state_->output_vio_ = TSVConnWrite(TSTransformOutputVConnGet(state_->vconn_), state_->vconn_, state_->output_buffer_reader_, 0);
 
     if (state_->output_vio_) {
